@@ -137,10 +137,8 @@ void WriteTransaction::removeDocument(quint64 id)
     failedIndexingDB.del(id);
 
     DocumentTimeDB::TimeInfo info = docTimeDB.get(id);
-    if (info.mTime) {
-        docTimeDB.del(id);
-        mtimeDB.del(info.mTime, id);
-    }
+    docTimeDB.del(id);
+    mtimeDB.del(info.mTime, id);
 
     docDataDB.del(id);
 }
@@ -162,11 +160,34 @@ void WriteTransaction::removeRecursively(quint64 parentId)
 
     const QVector<quint64> children = docUrlDB.getChildren(parentId);
     for (quint64 id : children) {
-        removeRecursively(id);
+        if (id) {
+            removeRecursively(id);
+        }
     }
     removeDocument(parentId);
 }
 
+bool WriteTransaction::removeRecursively(quint64 parentId, const std::function<bool(quint64)> &shouldDelete)
+{
+    DocumentUrlDB docUrlDB(m_dbis.idTreeDbi, m_dbis.idFilenameDbi, m_txn);
+
+    if (parentId && !shouldDelete(parentId)) {
+        return false;
+    }
+
+    bool isEmpty = true;
+    const QVector<quint64> children = docUrlDB.getChildren(parentId);
+    for (quint64 id : children) {
+        isEmpty &= removeRecursively(id, shouldDelete);
+    }
+    // refetch
+    if (isEmpty && docUrlDB.getChildren(parentId).isEmpty()) {
+        removeDocument(parentId);
+        return true;
+    }
+    return false;
+}
+
 void WriteTransaction::replaceDocument(const Document& doc, DocumentOperations operations)
 {
     DocumentDB documentTermsDB(m_dbis.docTermsDbi, m_txn);
